#!/bin/sh  -- # perl
eval 'exec ${PERLPATH}perl $0 ${1+"$@"}'
if 0;
#
# sssc2rdss.pl - convert flat-file SSSCs to PIDC format
#
# Istvan Bondar, 1999/02/24
#
# USAGE: sssc2rdss.pl outdir infillist
#   where outdir is the directory path for the output file
#	      infillist is the list of input filenames
#
#   infil:
#      header line:
#         sta, phase, region (one word, max 16 chars)
#      body lines:
#         lat, lon, SSSC, modeling error
#      where
#         sta is a stationcode,
#         phase is a phasename (Pn, Pg, Sn, Lg, P, etc),
#         region is a shortcut for a region, like nam or fenno
#
# OUTPUT: TT.{sta}.{phase}.reg.{region} - a PIDC compliant SSSC file 
#
# CHANGES: 
#    2001/05/10 
#       inserted extra comments
#       public release
#    2001/07/13
#       header info (sta,phase,region) is expected in the input file
#       only the input file is a command line parameter
#       comment or empty lines are allowed
#    2001/09/14
#       enabled to process multiple files.
#    2002/10/16
#       fixed problem in finding unique lats or lons.
#
#
# 
use strict;
$|=1;					# autoflush

print "Convert plain ascii SSSC file to CMR RDSS-format SSSC file\n\n";
print "                 Istvan Bondar, 2002\n\n";
print "    Science Application International Corporation\n";
print "            Center for Monitoring Research\n";
print "1300 N 17th Street, Suite 1450, Arlington VA 22209, USA\n";
print "    tel: (+1) 703 247-1831,  fax: (+1) 703 243-8950\n";
print "               e-mail: ibondar\@cmr.gov\n\n";
#
# check arguments, provide usage info and terminate if there are
#  incorrect number of arguments
#
if ( $#ARGV < 2 ) {
   print "Usage: sssc2rdss.pl outdir infillist\n";
   print "   where outdir is the directory path for the output file\n";
   print "         infillist is the list of input filenames\n";
   print "infil structure (comment lines are allowed):\n";
   print "   header line:\n";
   print "      sta, phase, region (one word, max 16 chars)\n";
   print "   body lines:\n";
   print "      lat, lon, SSSC, modeling error\n";
   print "   where\n";
   print "      sta is a stationcode,\n";
   print "      phase is a phasename (Pn, Pg, Sn, Lg, P, S, etc),\n";
   print "      region is a shortcut for a region, like nam or fenno.\n\n";
   print "Output: TT.{sta}.{phase}.reg.{region} - a PIDC compliant SSSC file\n";
   print "\n   Example: sssc2rdss.pl /config/TT/iasp91/SSSC *.sssc\n";
   exit -1;
}   
my ($outdir, @inlist) = @ARGV;
my ($fname, $i, $j, $k);
my (@LoL, @lats, @lons, $lat, $lon, $nlat, $nlon, %la, %lo, $x);
my ($sta, $phase, $regio, $ishead);
#
# process list of files
#
for my $infil (@inlist) {
#
# read input file
#
   $ishead = 1;
   @LoL = @lats = @lons = ();
   open(IN, "$infil") || die "Can't open $infil: $!\n"; 
   while ( <IN> ) {
      s/\s+$//;
      next if /^#/;
      next if /^$/;
      if ($ishead) {
          ($sta, $phase, $regio) = split;
          $regio = substr($regio, 0, 16) if (length($regio) > 16);
          $ishead = 0;
          next;
      }
      push @LoL, [ split ];
   }
   close(IN);
#
#  get unique latitudes and longitudes
#
   %lo = %la = ();
   $nlat = $nlon = 0;
   for $x ( @LoL ) {
      $lat = $x->[0];
      unless (exists $la{$lat}) {
         $la{$lat} = 1;
         $lats[$nlat++] = $lat;
      }
      $lon = $x->[1];
      unless (exists $lo{$lon}) {
         $lo{$lon} = 1;
         $lon = ($lon < -180.) ? $lon + 360 : ($lon > 180) ? $lon - 360: $lon;
         $lons[$nlon++] = $lon;
      }
   }
#
#  sort latitudes in descending order
#
   @lats = sort { $b <=> $a } @lats;
#
#  sort longitudes in ascending order
#
   @lons = sort { $a <=> $b } @lons;
#
# reorder SSSCs so that they start from the NW corner of the rectangle
# (i.e. sorted by descending latitudes and ascending longitudes)
#
   @LoL=sort { $b->[0] <=> $a->[0] or $a->[1] <=> $b->[1] } @LoL;
#
#  open RDSS-format output file for writing
#
   $fname = $outdir . "/TT." . $sta . "." . $phase . ".reg." . $regio;
   open(OU, ">$fname") || die "Can't open $fname: $!\n"; 
#
#  write header
#
   $j = "           iasp91   reg   0.000";
   print OU "# Station Phase    Source Region    Velocity Model   Level  StaCor  SedVel MEFac\n"; 
   print OU "# ======= ======== ================ ================ =====  ======  ====== =====\n";
   if ( $phase =~ /^[pP]/ ) {
      printf OU "  %-7s %-8s %-16s %s   5.800   1.0\n",
             $sta, $phase, $regio, $j;
   }
   else {
      printf OU "  %-7s %-8s %-16s %s   3.350   1.0\n",
             $sta, $phase, $regio, $j;
   }
   print OU "# Number of Lat/Lon samples, respectively:\n"; 
   printf OU "%5d%5d\n", $nlat, $nlon; 
   print OU "# Latitude samples (deg) from north to south:\n"; 
   for ($i = 0; $i < $nlat; $i++ ) {
      printf OU "%10.3f", $lats[$i];
   }
   print OU "\n";
   print OU "# Longitude samples (deg) from west to east:\n"; 
   for ($i = 0; $i < $nlon; $i++ ) {
      printf OU "%10.3f", $lons[$i];
   }
   print OU "\n";
#
# write SSSCs
#
   print OU "# SSSC's in map view (origin is NW corner):\n"; 
   $k=0;
   for ($i = 0; $i < $nlat; $i++ ) {
      for ($j = 0; $j < $nlon; $j++ ) {
         printf OU "%8.2f", $LoL[$k++][2];
      }
      print OU "\n";
   }
#
# write MEs
#
   print OU "# Modeling errors in map view (origin is NW corner):\n"; 
   $k=0;
   for ($i = 0; $i < $nlat; $i++ ) {
      for ($j = 0; $j < $nlon; $j++ ) {
         printf OU "%8.2f", $LoL[$k++][3];
      }
      print OU "\n";
   }
   close(OU);
   print "$infil done.\n";
}
